@@ -16,6 +16,7 @@ extern struct tseq tseq[];
 extern short e820_nr;
 extern char memsz_mode;
 void performance();
+extern volatile short cpu_mode;
 
 char save[2][POP_H][POP_W];
 char save2[2][POP2_H][POP2_W];
@@ -29,15 +30,15 @@ void get_config()
 	popup();
 	wait_keyup();
 	while(!flag) {
-		cprint(POP_Y+1,  POP_X+2, "Configuration:");
+		cprint(POP_Y+1,  POP_X+2, "Settings:");
 		cprint(POP_Y+3,  POP_X+6, "(1) Test Selection");
 		cprint(POP_Y+4,  POP_X+6, "(2) Address Range");
 		cprint(POP_Y+5,  POP_X+6, "(3) Memory Sizing");
 		cprint(POP_Y+6,  POP_X+6, "(4) Error Report Mode");
 		cprint(POP_Y+7,  POP_X+6, "(5) Show DMI Memory Info");
 		cprint(POP_Y+8,  POP_X+6, "(6) ECC Mode"); 
-		cprint(POP_Y+9,  POP_X+6, "(7) Restart w/Defaults");
-		cprint(POP_Y+10, POP_X+6, "(8) Redraw Screen");
+		cprint(POP_Y+9,  POP_X+6, "(7) CPU Selection Mode");
+		cprint(POP_Y+10, POP_X+6, "(8) Refresh Screen");
 		cprint(POP_Y+11, POP_X+6, "(9) Adv. Options");
 		cprint(POP_Y+12,POP_X+6,"(0) Continue");
 
@@ -68,9 +69,9 @@ void get_config()
 						bail++;
 					}
 					v->testsel = -1;
-					find_ticks();
+					find_ticks_for_pass();
 					sflag++;
-					cprint(LINE_INFO, COL_TST, "Std");
+					cprint(LINE_INFO, COL_INF3-3, "Std");
 					break;
 				case 3:
 					/* Skip test */
@@ -92,11 +93,11 @@ void get_config()
 						}
 						v->testsel = i;
 					}
-					find_ticks();
+					find_ticks_for_pass();
 					sflag++;
-					bail++;
-					cprint(LINE_INFO, COL_TST, "#");
-					dprint(LINE_INFO, COL_TST+1, i, 2, 1);
+                                        bail++;
+					cprint(LINE_INFO, COL_INF3-4, "  #");
+					dprint(LINE_INFO, COL_INF3-1, i, 1, 0);
 					break;
 				case 5:
 					if (v->testsel != 9) {
@@ -104,11 +105,10 @@ void get_config()
 						v->test = -1;
 					}
 					v->testsel = 9;
-                                        find_ticks();
+                                        find_ticks_for_pass();
                                         sflag++;
                                         bail++;
-                                        cprint(LINE_INFO, COL_TST, "#");
-                                        dprint(LINE_INFO, COL_TST+1, 9, 2, 1);
+					cprint(LINE_INFO, COL_INF3-2, "#9");
                                         break;
 				case 11:
 				case 57:
@@ -146,7 +146,7 @@ void get_config()
 						bail++;
 					}
 					adj_mem();
-					find_ticks();
+					find_ticks_for_pass();
 					sflag++;
 					break;
 				case 3:
@@ -162,11 +162,11 @@ void get_config()
 					page = getval(POP_Y+6, POP_X+9, 12);
 					if  (page - 1 >= v->plim_lower) {
 						v->plim_upper = page;
-						v->test--;
 						bail++;
+						v->test--;
 					}
 					adj_mem();
-					find_ticks();
+					find_ticks_for_pass();
 					sflag++;
 					break;
 				case 4:
@@ -177,7 +177,7 @@ void get_config()
 					v->test--;
 					bail++;
 					adj_mem();
-					find_ticks();
+					find_ticks_for_pass();
 					sflag++;
 					break;
 				case 11:
@@ -194,14 +194,10 @@ void get_config()
 			popclear();
 			cprint(POP_Y+1, POP_X+2, "Memory Sizing:");
 			cprint(POP_Y+3, POP_X+6, "(1) BIOS - Std");
-			if (e820_nr) {
-				cprint(POP_Y+4, POP_X+6, "(2) BIOS - All");
-				cprint(POP_Y+5, POP_X+6, "(3) Probe");
-				cprint(POP_Y+6, POP_X+6, "(0) Continue");
-				cprint(POP_Y+2+memsz_mode, POP_X+5, ">");
-			} else {
-				cprint(POP_Y+4, POP_X+6, "(3) Probe");
-				cprint(POP_Y+5, POP_X+6, "(0) Cancel");
+			cprint(POP_Y+4, POP_X+6, "(2) Probe");
+			cprint(POP_Y+5, POP_X+6, "(0) Continue");
+			cprint(POP_Y+2+memsz_mode, POP_X+5, ">");
+			if (!e820_nr) {
 				if (memsz_mode == SZ_MODE_BIOS) {
 					cprint(POP_Y+3, POP_X+5, ">");
 				} else {
@@ -217,11 +213,6 @@ void get_config()
 					restart();
 					break;
 				case 3:
-					memsz_mode = SZ_MODE_BIOS_RES;
-					wait_keyup();
-					restart();
-					break;
-				case 4:
 					memsz_mode = SZ_MODE_PROBE;
 					wait_keyup();
 					restart();
@@ -324,8 +315,39 @@ void get_config()
 			popclear();
 			break;
 		case 8:
+    			/* 7 - CPU Mode */
+			popclear();
+			cprint(POP_Y+1, POP_X+2, "CPU Selection Mode:");
+			cprint(POP_Y+3, POP_X+6, "(1) Single");
+			cprint(POP_Y+4, POP_X+6, "(2) Round Robin");
+			cprint(POP_Y+5, POP_X+6, "(3) Sequential");
+			cprint(POP_Y+6, POP_X+6, "(0) Cancel");
+			cprint(POP_Y+2+cpu_mode, POP_X+5, ">");
 			wait_keyup();
-			restart();
+			while(!sflag) {
+				switch(get_key()) {
+				case 2:
+					cpu_mode = CPM_SINGLE;
+					sflag++;
+					cprint(LINE_INFO+1,COL_INF3-6,"Single");
+					break;
+				case 3:
+					cpu_mode = CPM_RROBIN;
+					sflag++;
+					cprint(LINE_INFO+1,COL_INF3-6,"RRobin");
+					break;
+				case 4:
+					cpu_mode = CPM_SEQ;
+					sflag++;
+					cprint(LINE_INFO+1,COL_INF3-6,"   Seq");
+				case 11:
+				case 57:
+					/* 0/CR - Continue */
+					sflag++;
+					break;
+				}
+			}
+			popclear();
 			break;
 		case 9:
 			reprint_screen = 1;
@@ -377,6 +399,7 @@ void get_config()
         }
 }
 
+#ifdef NO
 void performance()
 { 
 	extern int l1_cache, l2_cache;
@@ -426,6 +449,7 @@ void performance()
 	while (get_key() == 0);
 	popclear();
 }
+#endif
 	
 
 void popup()
