@@ -8,6 +8,7 @@
  * http://www.x86-secret.com - http://www.memtest.org
  */
 
+#include "stddef.h"
 #include "test.h"
 #include "defs.h"
 #include "config.h"
@@ -21,8 +22,9 @@ extern short firmware;
 extern short dmi_initialized;
 extern int dmi_err_cnts[MAX_DMI_MEMDEVS];
 
-int l1_cache=0, l2_cache=0;
+int l1_cache=0, l2_cache=0, l3_cache=0;
 struct cpu_ident cpu_id;
+int tsc_invariable = 0;
 ulong st_low, st_high;
 ulong end_low, end_high;
 ulong cal_low, cal_high;
@@ -55,7 +57,12 @@ static void display_init(void)
 	for(i=0, pp=(char *)(SCREEN_ADR+1); i<TITLE_WIDTH; i++, pp+=2) {
 		*pp = 0x47;
 	}
-	cprint(0, 0, "      Memtest-86 v3.4       ");
+
+#ifdef SMP
+	cprint(0, 0, "    Memtest-86 v3.5 SMP     ");
+#else
+	cprint(0, 0, "      Memtest-86 v3.5       ");
+#endif
 
 	/* Do reverse video for the bottom display line */
 	for(i=0, pp=(char *)(SCREEN_ADR+1+(24 * 160)); i<80; i++, pp+=2) {
@@ -79,6 +86,24 @@ void init(void)
 
 	/* Setup the display */
 	display_init();
+	cprint(0, COL_MID,"Pass   %");
+	cprint(1, COL_MID,"Test   %");
+	cprint(2, COL_MID,"Test #");
+	cprint(3, COL_MID,"Testing: ");
+	cprint(3, COL_MID+31,"Using CPU: ");
+	cprint(4, COL_MID,"Pattern: ");
+	cprint(LINE_CPU+1, 0, "L1 Cache: Unknown ");
+	cprint(LINE_CPU+2, 0, "L2 Cache: Unknown ");
+     	cprint(LINE_CPU+3, 0, "L3 Cache: None  ");
+     	cprint(LINE_CPU+4, 0, "Memory  :                   |-------------------------------------------------");
+     	cprint(LINE_CPU+5, 0, "Chipset : ");
+	for(i=0; i < 5; i++) {
+		cprint(i, COL_MID-2, "| ");
+	}
+	cprint(LINE_INFO, 0,   "Time:   0:00:00    Cached:          Test_Sel:   Std    Pass:         0");
+	cprint(LINE_INFO+1, 0, "MemMap:            Iter:            CPU_Sel:           Errors:       0");
+	cprint(LINE_INFO+2, 0, "Rsvd_Mem:          Act_CPUs:        ECC_Mem:           ECC Err:      0");
+	footer();
 
 	/* Determine the memory map */
 	if ((firmware == FIRMWARE_UNKNOWN) && 
@@ -92,15 +117,16 @@ void init(void)
 	}
 
 	mem_size();
+     	aprint(LINE_CPU+4, 10, v->test_pages);
 
 	/* setup pci */
 	pci_init();
 
-	v->test = 0;
-	v->pass = 0;
-	v->msg_line = 0;
-	v->ecount = 0;
-	v->ecc_ecount = 0;
+        v->test = 0;
+        v->pass = 0;
+        v->msg_line = 0;
+        v->ecount = 0;
+        v->ecc_ecount = 0;
 	v->testsel = -1;
 	v->msg_line = LINE_SCROLL-1;
 	v->scroll_start = v->msg_line * 160;
@@ -128,12 +154,6 @@ void init(void)
 		}
 	}
 
-	cprint(LINE_CPU+1, 0, "L1 Cache: Unknown ");
-	cprint(LINE_CPU+2, 0, "L2 Cache: Unknown ");
-	cprint(LINE_CPU+3, 0, "Memory  : ");
-	aprint(LINE_CPU+3, 10, v->test_pages);
-	cprint(LINE_CPU+4, 0, "Chipset : ");
-
 	/*
 	 * Need to find out CPU type before seting up pci. This is
 	 * because AMD Opterons dont have a host bridge on dev 0.
@@ -148,27 +168,10 @@ void init(void)
 
 	if (v->rdtsc) {
 		cacheable();
-		cprint(LINE_TIME, COL_TIME+4, ":  :");
 	}
-	cprint(0, COL_MID,"Pass   %");
-	cprint(1, COL_MID,"Test   %");
-	cprint(2, COL_MID,"Test #");
-	cprint(3, COL_MID,"Testing: ");
-	cprint(4, COL_MID,"Pattern: ");
-	cprint(LINE_INFO-2, 0, " WallTime   Cached  RsvdMem   MemMap   Cache  ECC  Test  Pass  Errors ECC Errs");
-	cprint(LINE_INFO-1, 0, " ---------  ------  -------  --------  -----  ---  ----  ----  ------ --------");
-	cprint(LINE_INFO, COL_TST, "Std");
-	cprint(LINE_INFO, COL_PASS, "    0");
-	cprint(LINE_INFO, COL_ERR, "     0");
-	cprint(LINE_INFO+1, 0, " -----------------------------------------------------------------------------");
 
-	for(i=0; i < 5; i++) {
-		cprint(i, COL_MID-2, "| ");
-	}
-	footer();
 	v->printmode=PRINTMODE_SUMMARY;
 	v->numpatn=0;
-	find_ticks();
 }
 
 #define FLAT 0
@@ -512,6 +515,13 @@ void cpu_type(void)
 				}
 				off = 18;
 				break;
+			case 2:
+				l3_cache = (cpu_id.cache_info[15] << 8);
+     				l3_cache += (cpu_id.cache_info[14] >> 2);
+     				l3_cache *= 512;
+     				cprint(LINE_CPU, 0, "AMD K10 CPU @");
+				off = 13;				
+				break;
 			case 3:
 			case 11:
 				cprint(LINE_CPU, 0, "Athlon 64 X2");
@@ -609,18 +619,20 @@ void cpu_type(void)
 			case 0x6:
 			case 0xa:
 			case 0x66:
-				l1_cache += 8;
+				l1_cache = 8;
 				break;
 			case 0x8:
 			case 0xc:
 			case 0x67:
 			case 0x60:
-				l1_cache += 16;
+				l1_cache = 16;
 				break;
+			case 0x9:
+			case 0xd:
 			case 0x68:
 			case 0x2c:
 			case 0x30:
-				l1_cache += 32;
+				l1_cache = 32;
 				break;
 			case 0x40:
 				l2_cache = 0;
@@ -634,10 +646,12 @@ void cpu_type(void)
 			case 0x3a:
 				l2_cache = 192;
 				break;
+			case 0x21:
 			case 0x42:
 			case 0x7a:
 			case 0x82:
 			case 0x3c:
+			case 0x3f:
 				l2_cache = 256;
 				break;
 			case 0x3d:
@@ -649,6 +663,7 @@ void cpu_type(void)
 			case 0x86:
 			case 0x3e:
 			case 0x7f:
+			case 0x80:
 				l2_cache = 512;
 				break;
 			case 0x44:
@@ -663,9 +678,34 @@ void cpu_type(void)
 			case 0x85:
 				l2_cache = 2048;
 				break;
+			case 0x48:
+				l2_cache = 3072;
+				break;
 			case 0x49:
 				l2_cache = 4096;
 				break;
+			case 0x4e:
+				l2_cache = 6144;
+				break;
+			case 0xd1:
+			case 0xd6:
+				l3_cache = 1024;
+				break;
+			case 0xd2:
+			case 0xd7:
+			case 0xdc:
+			case 0xe2:
+				l3_cache = 2048;
+				break;
+			case 0xd8:
+			case 0xdd:
+			case 0xe3:
+				l3_cache = 4096;
+				break;
+			case 0xde:
+			case 0xe4:
+				l3_cache = 8192;
+				break;	
 			}
 		}
 
@@ -706,6 +746,11 @@ void cpu_type(void)
 				off = 10;
 				break;
 			case 5:
+			    if (((cpu_id.ext >> 16) & 0xF) != 0) {
+				cprint(LINE_CPU, 0, "Intel EP80579");
+				if (l2_cache == 0) { l2_cache = 256; }
+				off = 13;
+			   } else {
 				if (l2_cache == 0) {
 					cprint(LINE_CPU, 0, "Celeron");
 					off = 7;
@@ -714,30 +759,42 @@ void cpu_type(void)
 					off = 10;
 				}
 				break;
+			   }
 			case 6:
-				if (l2_cache == 128) {
+				if (((cpu_id.ext >> 16) & 0xF) != 0) {
+     					tsc_invariable = 1;
+    					cprint(LINE_CPU, 0, "Intel Nehalem");
+    					off = 13;
+    				} else {
+				  if (l2_cache == 128) {
 					cprint(LINE_CPU, 0, "Celeron");
 					off = 7;
-				} else {
-				    if (cpu_id.step == 0 || cpu_id.step == 5) {
-					cprint(LINE_CPU, 0, "Celeron-A");
-					off = 9;
-				    } else {
+				  } else {
 					cprint(LINE_CPU, 0, "Pentium II");
 					off = 10;
-				    }
+				  }
 				}
 				break;
 			case 7:
 			case 8:
-			case 10:
 			case 11:
-				if (l2_cache == 128) {
+				if (((cpu_id.ext >> 16) & 0xF) != 0) {
+    					tsc_invariable = 1;
+    					if (l2_cache < 1024) {
+    						cprint(LINE_CPU, 0, "Celeron");
+    						off = 7;
+    					} else {
+    						cprint(LINE_CPU, 0, "Intel Core 2");
+    						off = 12;
+    					}
+				} else {
+				   if (l2_cache == 128) {
 					cprint(LINE_CPU, 0, "Celeron");
 					off = 7;
-				} else {
+				   } else {
 					cprint(LINE_CPU, 0, "Pentium III");
 					off = 11;
+				   }
 				}
 				break;
 			case 9:
@@ -748,6 +805,21 @@ void cpu_type(void)
 				}
 				off = 16;
 				break;
+     			case 10:
+				if (((cpu_id.ext >> 16) & 0xF) != 0) {
+				    tsc_invariable = 1;
+				    cprint(LINE_CPU, 0, "Intel Core i7");
+				    off = 13;
+				} else {
+				    cprint(LINE_CPU, 0, "Pentium III Xeon");
+				    off = 16;
+				}					
+				break;
+			case 12:
+				l1_cache = 24;
+				cprint(LINE_CPU, 0, "Atom (0.045)");
+				off = 12;
+				break;					
 			case 13:
 				if (l2_cache == 1024) {
 					cprint(LINE_CPU, 0, "Celeron M (0.09)");
@@ -757,12 +829,24 @@ void cpu_type(void)
 				off = 16;
 				break;
 			case 14:
-				cprint(LINE_CPU, 0, "Intel Core");
-				off = 10;
+				if (((cpu_id.ext >> 16) & 0xF) != 0) {
+					tsc_invariable = 1;
+					cprint(LINE_CPU, 0, "Intel Core i5");
+					off = 13;
+				} else {
+					cprint(LINE_CPU, 0, "Intel Core");
+					off = 10;
+				}
 				break;				
 			case 15:
-				cprint(LINE_CPU, 0, "Intel Core 2");
-				off = 12;
+				if (l2_cache == 1024) {
+					cprint(LINE_CPU, 0, "Pentium E");
+					off = 9;
+				} else {
+					cprint(LINE_CPU, 0, "Intel Core 2");
+					off = 12;
+				}
+				tsc_invariable = 1;
 				break;
 			}
 			break;
@@ -836,23 +920,44 @@ void cpu_type(void)
 	/* VIA/Cyrix/Centaur Processors with CPUID */
 	case 'C':
 		if ( cpu_id.vend_id[1] == 'e' ) {	/* CentaurHauls */
+			l1_cache = cpu_id.cache_info[3] + cpu_id.cache_info[7];
+			l2_cache = cpu_id.cache_info[11];
 			switch(cpu_id.type){
 			case 5:
 				cprint(LINE_CPU, 0, "Centaur 5x86");
 				off = 12;
 				break;
 			case 6: // VIA C3
-				if (cpu_id.step < 8) {
+				switch(cpu_id.model){
+				default:
+				    if (cpu_id.step < 8) {
 					cprint(LINE_CPU, 0, "VIA C3 Samuel2");
 					off = 14;
-				} else {
-					cprint(LINE_CPU, 0, "Via C3 Eden");
+				    } else {
+					cprint(LINE_CPU, 0, "VIA C3 Eden");
 					off = 11;
-				}
+				    }
 				break;
+				case 10:
+					cprint(LINE_CPU, 0, "VIA C7 (C5J)");
+					l1_cache = 64;
+					l2_cache = 128;
+					off = 16;
+					break;
+				case 13:
+					cprint(LINE_CPU, 0, "VIA C7 (C5R)");
+					l1_cache = 64;
+					l2_cache = 128;
+					off = 12;
+					break;
+				case 15:
+					cprint(LINE_CPU, 0, "VIA Isaiah (CN)");
+					l1_cache = 64;
+					l2_cache = 128;
+					off = 15;
+					break;
+				}
 			}
-			l1_cache = cpu_id.cache_info[3] + cpu_id.cache_info[7];
-			l2_cache = cpu_id.cache_info[11];
 		} else {				/* CyrixInstead */
 			switch(cpu_id.type) {
 			case 5:
@@ -938,7 +1043,7 @@ void cpu_type(void)
 		cprint(LINE_CPU+1, 0, "L1 Cache:     K  ");
 		dprint(LINE_CPU+1, 11, l1_cache, 3, 0);
 		if ((speed=memspeed((ulong)mapping(0x100),
-				(l1_cache / 4) * 1024, 50, MS_COPY))) {
+				(l1_cache / 4) * 1024, 200, MS_COPY))) {
 			cprint(LINE_CPU+1, 16, "       MB/s");
 			dprint(LINE_CPU+1, 16, speed, 6, 0);
 		}
@@ -957,24 +1062,40 @@ void cpu_type(void)
 		} else {
 			i = l1_cache;
 		}
-		if ((speed=memspeed((ulong)mapping(0x100), i*1024, 50,
+		if ((speed=memspeed((ulong)mapping(0x100), i*1024, 200,
 				MS_COPY))) {
 			cprint(LINE_CPU+2, 16, "       MB/s");
 			dprint(LINE_CPU+2, 16, speed, 6, 0);
 		}
 	}
+	/* Print out L3 cache info */
+	/* We measure the L3 cache speed by using a block size that is */
+	/* 2X the size of the L2 cache. */
 
-	/* Determine memory speed.  To find the memory speed we use */
-	/* A block size that is 5x the sum of the L1 and L2 caches */
-	i = (l2_cache + l1_cache) * 5;
+	if (l3_cache) {
+		cprint(LINE_CPU+3, 0, "L3 Cache:     K  ");
+    		dprint(LINE_CPU+3, 10, l3_cache, 4, 0);
+    		dprint(LINE_CPU+3, 10, l3_cache, 4, 0);
+    
+    		i = l2_cache*2;
+    
+    		if ((speed=memspeed((ulong)mapping(0x100), i*1024, 150, MS_COPY))) {
+    			cprint(LINE_CPU+3, 16, "       MB/s");
+    			dprint(LINE_CPU+3, 16, speed, 6, 0);
+    		}
+    	}
+    
+    	/* Determine memory speed.  To find the memory speed we use */
+    	/* A block size that is 5x the sum of the L1, L2 & L3 caches */
+    	i = (l3_cache + l2_cache + l1_cache) * 5;
 
 	/* Make sure that we have enough memory to do the test */
 	if ((1 + (i * 2)) > (v->plim_upper << 2)) {
 		i = ((v->plim_upper <<2) - 1) / 2;
 	}
-	if((speed = memspeed((ulong)mapping(0x100), i*1024, 40, MS_COPY))) {
-		cprint(LINE_CPU+3, 16, "       MB/s");
-		dprint(LINE_CPU+3, 16, speed, 6, 0);
+	if((speed = memspeed((ulong)mapping(0x100), i*1024, 50, MS_COPY))) {
+		cprint(LINE_CPU+4, 16, "       MB/s");
+		dprint(LINE_CPU+4, 16, speed, 6, 0);
 	}
 
 	/* Record the sarting time */
@@ -1020,7 +1141,7 @@ static void cacheable(void)
 			pspeed = (ulong)((float)speed * 0.7);
 		}
 	}
-	aprint(LINE_INFO, COL_CACHE_TOP, cached);
+	aprint(LINE_INFO, COL_INF2-5, cached);
 	/* Ensure the default set of pages are mapped */
 	map_page(0);
 	map_page(0x80000);
@@ -1061,6 +1182,7 @@ static int cpuspeed(void)
 		return(-1);
 	}
 	v->clks_msec = end_low/50;
+	if (tsc_invariable) end_low = correct_tsc(end_low);
 	return(v->clks_msec);
 }
 
@@ -1072,6 +1194,8 @@ ulong memspeed(ulong src, ulong len, int iter, int type)
 	ulong wlen;
 	int i;
 
+	if (len == 0) return(0);
+
 	dst = src + len;
 	wlen = len / 4;  /* Length is bytes */
 
@@ -1209,5 +1333,43 @@ ulong memspeed(ulong src, ulong len, int iter, int type)
 	}
 
 	/* Convert to kbytes/sec */
+	if (tsc_invariable) end_low = correct_tsc(end_low);
 	return((v->clks_msec)/end_low);
 }
+
+#define rdmsr(msr,val1,val2) \
+	__asm__ __volatile__("rdmsr" \
+		  : "=a" (val1), "=d" (val2) \
+		  : "c" (msr))
+
+ulong correct_tsc(ulong el_org)
+{
+	float coef_now, coef_max;
+	int msr_lo, msr_hi, is_xe;
+	
+	rdmsr(0x198, msr_lo, msr_hi);
+	is_xe = (msr_lo >> 31) & 0x1;		
+	
+	if(is_xe){
+		rdmsr(0x198, msr_lo, msr_hi);
+		coef_max = ((msr_hi >> 8) & 0x1F);	
+		if ((msr_hi >> 14) & 0x1) { coef_max = coef_max + 0.5f; }
+	} else {
+		rdmsr(0x17, msr_lo, msr_hi);
+		coef_max = ((msr_lo >> 8) & 0x1F);
+		if ((msr_lo >> 14) & 0x1) { coef_max = coef_max + 0.5f; }
+	}
+	
+	if((cpu_id.feature_flag >> 7) & 1) {
+		rdmsr(0x198, msr_lo, msr_hi);
+		coef_now = ((msr_lo >> 8) & 0x1F);
+		if ((msr_lo >> 14) & 0x1) { coef_now = coef_now + 0.5f; }
+	} else {
+		rdmsr(0x2A, msr_lo, msr_hi);
+		coef_now = (msr_lo >> 22) & 0x1F;
+	}
+	if(coef_max && coef_now) {
+		el_org = (ulong)(el_org * coef_now / coef_max);
+	}
+	return el_org;
+}
