@@ -8,9 +8,23 @@
 #include "test.h"
 #include "config.h"
 #include "screen_buffer.h"
+#include "smp.h"
 
 int slock = 0, lsr = 0;
 short serial_cons = SERIAL_CONSOLE_DEFAULT;
+#if SERIAL_TTY != 0 && SERIAL_TTY != 1
+#error Bad SERIAL_TTY. Only ttyS0 and ttyS1 are supported.
+#endif
+short serial_tty = SERIAL_TTY;
+const short serial_base_ports[] = {0x3f8, 0x2f8};
+
+#if ((115200%SERIAL_BAUD_RATE) != 0)
+#error Bad default baud rate
+#endif
+int serial_baud_rate = SERIAL_BAUD_RATE;
+unsigned char serial_parity = 0;
+unsigned char serial_bits = 8;
+
 char buf[18];
 
 struct ascii_map_str {
@@ -40,8 +54,16 @@ char *codes[] = {
 	" Mch_Chk",
 	"SIMD FPE"
 };
+inline void reboot(void)
+{
+	/* tell the BIOS to do a warm start */
+	*((unsigned short *)0x472) = 0x1234;
+	outb(0xfe,0x64);
+}
 
 struct eregs {
+	ulong ss;
+	ulong ds;
 	ulong esp;
 	ulong ebp;
 	ulong esi;
@@ -69,6 +91,20 @@ int memcmp(const void *s1, const void *s2, ulong count)
 	return 0;
 }
 
+int strncmp(const char *s1, const char *s2, ulong n) {
+	signed char res = 0;
+	while (n) {
+		res = *s1 - *s2;
+		if (res != 0)
+			return res;
+		if (*s1 == '\0')
+			return 0;
+		++s1, ++s2;
+		--n;
+	}
+	return res;
+}
+
 void *memmove(void *dest, const void *src, ulong n)
 {
 	long i;
@@ -88,6 +124,50 @@ void *memmove(void *dest, const void *src, ulong n)
 	return dest;
 }
 
+char toupper(char c)
+{
+	if (c >= 'a' && c <= 'z')
+		return c + 'A' -'a';
+	else
+		return c;
+}
+
+int isdigit(char c)
+{
+	return c >= '0' && c <= '9';
+}
+
+int isxdigit(char c)
+{
+	return isdigit(c) || (toupper(c) >= 'A' && toupper(c) <= 'F'); }
+
+unsigned long simple_strtoul(const char *cp, char **endp, unsigned int base) {
+	unsigned long result = 0, value;
+
+	if (!base) {
+		base = 10;
+		if (*cp == '0') {
+			base = 8;
+			cp++;
+			if (toupper(*cp) == 'X' && isxdigit(cp[1])) {
+				cp++;
+				base = 16;
+			}
+		}
+	} else if (base == 16) {
+		if (cp[0] == '0' && toupper(cp[1]) == 'X')
+			cp += 2;
+	}
+	while (isxdigit(*cp) &&
+		(value = isdigit(*cp) ? *cp-'0' : toupper(*cp)-'A'+10) < base) {
+		result = result*base + value;
+		cp++;
+	}
+	if (endp)
+		*endp = (char *)cp;
+	return result;
+}
+
 /*
  * Scroll the error message area of the screen as needed
  * Starts at line LINE_SCROLL and ends at line 23
@@ -140,7 +220,7 @@ void clear_scroll(void)
 }
 
 /*
- * Print characters (n screen
+ * Print characters on screen
  */
 void cprint(int y, int x, const char *text)
 {
@@ -183,6 +263,19 @@ void reverse(char s[])
     s[j] = c;
   }
 }
+void memcpy (void *dst, void *src, int len)
+{
+	char *s = (char*)src;
+	char *d = (char*)dst;
+	int i;
+
+	if (len <= 0) {
+		return;
+	}
+	for (i = 0 ; i < len; i++) {
+		*d++ = *s++;
+	} 
+}
 
 /*
  * Print a people friendly address
@@ -357,6 +450,7 @@ void inter(struct eregs *trap_regs)
 	int i, line;
 	unsigned char *pp;
 	ulong address = 0;
+	int my_cpu_num = smp_my_cpu_num();
 
 	/* Get the page fault address */
 	if (trap_regs->vect == 14) {
@@ -378,7 +472,8 @@ void inter(struct eregs *trap_regs)
         }
 	line = LINE_SCROLL-2;
 
-	cprint(line, 0, "Unexpected Interrupt - Halting");
+	cprint(line, 0, "Unexpected Interrupt - Halting CPU");
+	dprint(line, COL_MID + 4, my_cpu_num, 2, 1);
 	cprint(line+2, 0, " Type: ");
 	if (trap_regs->vect <= 19) {
 		cprint(line+2, 7, codes[trap_regs->vect]);
@@ -393,6 +488,10 @@ void inter(struct eregs *trap_regs)
 	hprint(line+5, 7, trap_regs->eflag);
 	cprint(line+6, 0, " Code: ");
 	hprint(line+6, 7, trap_regs->code);
+	cprint(line+7, 0, "   DS: ");
+	hprint(line+7, 7, trap_regs->ds);
+	cprint(line+8, 0, "   SS: ");
+	hprint(line+8, 7, trap_regs->ss);
 	if (trap_regs->vect == 14) {
 		/* Page fault address */
 		cprint(line+7, 0, " Addr: ");
@@ -440,17 +539,14 @@ void set_cache(int val)
 	extern struct cpu_ident cpu_id;
 	/* 386's don't have a cache */
 	if ((cpu_id.cpuid < 1) && (cpu_id.type == 3)) {
-		cprint(LINE_INFO, COL_CACHE, "none");
 		return;
 	}
 	switch(val) {
 	case 0:
 		cache_off();	
-		cprint(LINE_INFO, COL_CACHE, "off");
 		break;
 	case 1:
 		cache_on();
-		cprint(LINE_INFO, COL_CACHE, " on");
 		break;
 	}
 }
@@ -487,11 +583,7 @@ void check_input(void)
 		case 1:	
 			/* "ESC" key was pressed, bail out.  */
 			cprint(LINE_RANGE, COL_MID+23, "Halting... ");
-
-			/* tell the BIOS to do a warm start */
-			*((unsigned short *)0x472) = 0x1234;
-			cache_on();
-			outb(0xfe,0x64);
+			reboot();
 			break;
 		case 46:
 			/* c - Configure */
@@ -625,21 +717,7 @@ ulong getval(int x, int y, int result_shift)
 	shift -= result_shift;
 
 	/* Compute our current value */
-	val = 0;
-	for(i = (base == 16)? 2: 0; i < n; i++) {
-		unsigned long digit = 0;
-		if ((buf[i] >= '0') && (buf[i] <= '9')) {
-			digit = buf[i] - '0';
-		}
-		else if ((buf[i] >= 'a') && (buf[i] <= 'f')) {
-			digit = buf[i] - 'a' + 10;
-		}
-		else {
-				/* It must be a suffix byte */
-			break;
-		}
-		val = (val * base) + digit;
-	}
+	val = simple_strtoul(buf, 0, base);
 	if (shift > 0) {
 		if (shift >= 32) {
 			val = 0xffffffff;
@@ -675,20 +753,11 @@ void ttyprint(int y, int x, const char *p)
 	serial_echo_print(p);
 }
 
-#if defined(SERIAL_BAUD_RATE)
-
-#if ((115200%SERIAL_BAUD_RATE) != 0)
-#error Bad ttys0 baud rate
-#endif
-
-#define SERIAL_DIV     (115200/SERIAL_BAUD_RATE)
-
-#endif /* SERIAL_BAUD_RATE */
-
 void serial_echo_init(void)
 {
-	int comstat, hi, lo;
-	
+	int comstat, hi, lo, serial_div;
+	unsigned char lcr;	
+
 	/* read the Divisor Latch */
 	comstat = serial_echo_inb(UART_LCR);
 	serial_echo_outb(comstat | UART_LCR_DLAB, UART_LCR);
@@ -697,13 +766,13 @@ void serial_echo_init(void)
 	serial_echo_outb(comstat, UART_LCR);
 
 	/* now do hardwired init */
-	serial_echo_outb(0x03, UART_LCR); /* No parity, 8 data bits, 1 stop */
-#if defined(SERIAL_BAUD_RATE)
-	serial_echo_outb(0x83, UART_LCR); /* Access divisor latch */
-	serial_echo_outb(SERIAL_DIV & 0xff, UART_DLL);  /* baud rate divisor */
-	serial_echo_outb((SERIAL_DIV>> 8) & 0xff, UART_DLM);
-	serial_echo_outb(0x03, UART_LCR); /* Done with divisor */
-#endif
+	lcr = serial_parity | (serial_bits - 5);
+	serial_echo_outb(lcr, UART_LCR); /* No parity, 8 data bits, 1 stop */
+	serial_div = 115200 / serial_baud_rate;
+	serial_echo_outb(0x80|lcr, UART_LCR); /* Access divisor latch */
+	serial_echo_outb(serial_div & 0xff, UART_DLL);  /* baud rate divisor */
+	serial_echo_outb((serial_div >> 8) & 0xff, UART_DLM);
+	serial_echo_outb(lcr, UART_LCR); /* Done with divisor */
 
 	/* Prior to disabling interrupts, read the LSR and RBR
 	 * registers */
@@ -928,3 +997,91 @@ void wait_keyup( void ) {
 		}
 	}
 }
+
+/*
+ * Handles "console=<param>" command line option
+ *
+ * Examples of accepted params:
+ *   ttyS0
+ *   ttyS1
+ *   ttyS0,115200
+ *   ttyS0,9600e8
+ */
+void serial_console_setup(char *param)
+{
+	char *option, *end;
+	unsigned long tty;
+	unsigned long baud_rate;
+	unsigned char parity, bits;
+
+	if (strncmp(param, "ttyS", 4))
+		return;   /* not a serial port */
+
+	param += 4;
+
+	tty = simple_strtoul(param, &option, 10);
+
+	if (option == param)
+		return;   /* there were no digits */
+
+	if (tty > 1)
+		return;   /* only ttyS0 and ttyS1 supported */
+
+	if (*option == '\0' || *option == ' ')
+		goto save_tty; /* no options given, just ttyS? */
+
+	if (*option != ',')
+		return;  /* missing the comma separator */
+
+	/* baud rate must follow */
+	option++;
+	baud_rate = simple_strtoul(option, &end, 10);
+
+	if (end == option)
+		return;  /* no baudrate after comma */
+
+	if (baud_rate == 0 || (115200 % baud_rate) != 0)
+		return;  /* wrong baud rate */
+
+	if (*end == '\0' || *end == ' ')
+		goto save_baud_rate;  /* no more options given */
+
+	switch (toupper(*end)) {
+		case 'N':
+			parity = 0;
+			break;
+		case 'O':
+			parity = UART_LCR_PARITY;
+			break;
+		case 'E':
+			parity = UART_LCR_PARITY | UART_LCR_EPAR;
+			break;
+		default:
+			/* Unknown parity */
+			return;
+	}
+
+	end++;
+	if (*end == '\0' || *end == ' ')
+		goto save_parity;
+
+	/* word length (bits) */
+	if (*end < '7' || *end > '8')
+		return;  /* invalid number of bits */
+
+	bits = *end - '0';
+
+	end++;
+
+	if (*end != '\0' || *end != ' ')
+		return;  /* garbage at the end */
+
+	serial_bits = bits;
+	save_parity:
+	serial_parity = parity;
+	save_baud_rate:
+	serial_baud_rate = (int) baud_rate;
+  save_tty:
+	serial_tty = (short) tty;
+	serial_cons = 1;
+}
