@@ -1,10 +1,11 @@
-/* test.c - MemTest-86  Version 3.3
+/* test.c - MemTest-86  Version 3.4
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
  */
 #include "test.h"
 #include "config.h"
+#include "smp.h"
 
 extern int segs, bail;
 extern volatile ulong *p;
@@ -42,7 +43,7 @@ void addr_tst1()
 		/* Now write pattern compliment */
 		p1 = ~p1;
 		end = v->map[segs-1].end;
-		for (i=0; i<1000; i++) {
+		for (i=0; i<100; i++) {
 			mask = 4;
 			do {
 				pt = (ulong *)((ulong)p | mask);
@@ -86,7 +87,7 @@ void addr_tst1()
 				*p = p1;
 
 				p1 = ~p1;
-				for (i=0; i<200; i++) {
+				for (i=0; i<50; i++) {
 					mask = 4;
 					do {
 						pt = (ulong *)
@@ -1294,7 +1295,7 @@ void bit_fade()
 			end = v->map[j].end;
 			pe = start;
 			p = start;
-			for (p=end; p<end; p++) {
+			for (p=start; p<end; p++) {
 				*p = p1;
 			}
 			do_tick();
@@ -1309,7 +1310,7 @@ void bit_fade()
 			end = v->map[j].end;
 			pe = start;
 			p = start;
-			for (p=end; p<end; p++) {
+			for (p=start; p<end; p++) {
  				if ((bad=*p) != p1) {
 					error((ulong*)p, p1, bad);
 				}
@@ -1354,8 +1355,8 @@ void sleep(int n)
 
 		/* Display the elapsed time on the screen */
 		i = t % 60;
-		dprint(LINE_TIME, COL_TIME+9, i%10, 1, 0);
-		dprint(LINE_TIME, COL_TIME+8, i/10, 1, 0);
+		dprint(LINE_INFO, COL_INF1-1, i%10, 1, 0);
+		dprint(LINE_INFO, COL_INF1-2, i/10, 1, 0);
 		if (i != ip) {
 			do_tick();
 			BAILR
@@ -1363,9 +1364,9 @@ void sleep(int n)
 		}
 		t /= 60;
 		i = t % 60;
-		dprint(LINE_TIME, COL_TIME+6, i % 10, 1, 0);
-		dprint(LINE_TIME, COL_TIME+5, i / 10, 1, 0);
+		dprint(LINE_INFO, COL_INF1-4, i % 10, 1, 0);
+		dprint(LINE_INFO, COL_INF1-5, i / 10, 1, 0);
 		t /= 60;
-		dprint(LINE_TIME, COL_TIME, t, 4, 0);
+		dprint(LINE_INFO, COL_INF1-10, t, 4, 0);
 	}
 }
