@@ -1,10 +1,11 @@
-/* test.h - MemTest-86  Version 3.3
+/* test.h - MemTest-86  Version 3.4
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
  */
 
-
+#ifndef _TEST_H_
+#define _TEST_H_
 #define MAX_DMI_MEMDEVS 16
 #define E88     0x00
 #define E801    0x04
@@ -36,51 +37,52 @@ struct mem_info_t {
 };
 
 typedef unsigned long ulong;
-#define SPINSZ		0x1000000 /* 1mb */
+#define SPINSZ		0x2000000
 #define MOD_SZ		20
-#define BAILOUT		if (bail) goto skip_test;
+#define BAILOUT		if (bail) return(1);
 #define BAILR		if (bail) return;
 
+#define DMI_SEARCH_START  0x0000F000
+#define DMI_SEARCH_LENGTH 0x000F0FFF
+#define MAX_DMI_MEMDEVS 16
+
 #define RES_START	0xa0000
 #define RES_END		0x100000
 #define SCREEN_ADR	0xb8000
 #define SCREEN_END_ADR  (SCREEN_ADR + 80*25*2)
 
 #define TITLE_WIDTH	28
-#define LINE_TIME	10
-#define COL_TIME	0
+#define LINE_CPU	1
 #define LINE_TST	2
 #define LINE_RANGE	3
-#define LINE_CPU	1
-#define COL_MID		30
 #define LINE_PAT        4
-#define COL_PAT		41
-#define LINE_INFO	10
-#define COL_CACHE_TOP   13
-#define COL_RESERVED    22
-#define COL_MMAP	29
-#define COL_CACHE	40
-#define COL_ECC		46
-#define COL_TST		52
-#define COL_PASS	56
-#define COL_ERR		63
-#define COL_ECC_ERR	72
+#define LINE_STATUS	7
+#define LINE_INFO	9
 #define LINE_HEADER	12
 #define LINE_SCROLL	14
-#define BAR_SIZE	(78-COL_MID-9)
 #define LINE_MSG	18
+#define COL_INF1        15
+#define COL_INF2        32
+#define COL_INF3        51
+#define COL_INF4        70
+#define COL_MID		30
+#define COL_PAT		41
+#define BAR_SIZE	(78-COL_MID-9)
 #define COL_MSG		18
 
 #define POP_W	34
 #define POP_H	15
-#define POP_X	16
+#define POP_X	11
 #define POP_Y	8
 #define POP2_W  74
 #define POP2_H  21
 #define POP2_X  3
 #define POP2_Y  2
 
-#define NULL	0
+/* CPU mode types */
+#define CPM_SINGLE 1
+#define CPM_RROBIN 2
+#define CPM_SEQ    3
 
 /* memspeed operations */
 #define MS_COPY		1
@@ -88,12 +90,12 @@ typedef unsigned long ulong;
 #define MS_READ		3
 
 #define SZ_MODE_BIOS		1
-#define SZ_MODE_BIOS_RES	2
-#define SZ_MODE_PROBE		3
+#define SZ_MODE_PROBE		2
 
 #define getCx86(reg) ({ outb((reg), 0x22); inb(0x23); })
 int memcmp(const void *s1, const void *s2, ulong count);
 void *memmove(void *dest, const void *src, ulong n);
+int strncmp(const char *s1, const char *s2, ulong n);
 int query_linuxbios(void);
 int query_pcbios(void);
 int insertaddress(ulong);
@@ -101,6 +103,7 @@ void printpatn(void);
 void printpatn(void);
 void itoa(char s[], int n); 
 void reverse(char *p);
+void serial_console_setup(char *param);
 void serial_echo_init(void);
 void serial_echo_print(const char *s);
 void ttyprint(int y, int x, const char *s);
@@ -168,6 +171,7 @@ void *mapping(unsigned long page_address);
 void *emapping(unsigned long page_address);
 ulong memspeed(ulong src, ulong len, int iter, int type);
 unsigned long page_of(void *ptr);
+ulong correct_tsc(ulong el_org);
 
 #define PRINTMODE_SUMMARY   0
 #define PRINTMODE_ADDRESSES 1
@@ -202,29 +206,6 @@ static inline void cache_on(void)
 		"pop  %eax\n\t");
 }
 
-static inline void reboot(void)
-{
-        asm(
-		"movl %cr0,%eax\n\t"
-       		"andl  $0x00000011,%eax\n\t"
-       		"orl   $0x60000000,%eax\n\t"
-       		"movl  %eax,%cr0\n\t"
-       		"movl  %eax,%cr3\n\t"
-		"movl  %cr0,%ebx\n\t"
-		"andl  $0x60000000,%ebx\n\t"
-		"jz    f\n\t"
-		".byte 0x0f,0x09\n\t"	/* Invalidate and flush cache */
-		"f: andb  $0x10,%al\n\t"
-		"movl  %eax,%cr0\n\t"
-		"movw $0x0010,%ax\n\t"
-		"movw %ax,%ds\n\t"
-		"movw %ax,%es\n\t"
-		"movw %ax,%fs\n\t"
-		"movw %ax,%gs\n\t"
-		"movw %ax,%ss\n\t"
-		"ljmp  $0xffff,$0x0000\n\t");
-}
-
 struct mmap {
 	ulong pbase_addr;
 	ulong *start;
@@ -240,7 +221,6 @@ struct tseq {
 	short cache;
 	short pat;
 	short iter;
-	short ticks;
 	short errors;
 	char *msg;
 };
@@ -256,6 +236,7 @@ struct cpu_ident {
 	unsigned char cache_info[16];
 	long pwrcap;
 	long ext;
+	long feature_flag;
 };
 
 struct xadr {
@@ -283,7 +264,7 @@ struct err_info {
 
 /* Define common variables accross relocations of memtest86 */
 struct vars {
-	int test;
+	volatile int test;
 	int pass;
 	int msg_line;
 	int ecount;
@@ -326,3 +307,4 @@ extern unsigned char _size, _pages;
 extern struct mem_info_t mem_info;
 
 #endif /* __ASSEMBLY__ */
+#endif /* _TEST_H_ */
